@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2017, Xilinx, Inc. All rights reserved.
+ *  Copyright (C) 2022 Advanced Micro Devices, Inc. All rights reserved.
  *  Authors:
  *    Sonal Santan <sonal.santan@xilinx.com>
  *
@@ -36,7 +37,7 @@
 
 /**
  * DOC: A GEM style driver for Xilinx PCIe based accelerators
- * This file defines ioctl command codes and associated structures for interacting with
+ * File *xocl_ioctl.h* defines ioctl command codes and associated structures for interacting with
  * *xocl* PCI driver for Xilinx FPGA platforms.
  *
  * Device memory allocation is modeled as buffer objects (bo). For each bo driver tracks the host pointer
@@ -72,20 +73,30 @@
  * 7    Read back data in bo backing storage   DRM_IOCTL_XOCL_PREAD_BO        drm_xocl_pread_bo
  * 8    Open/close a context on a compute unit DRM_IOCTL_XOCL_CTX             drm_xocl_ctx
  *      on the device
- * 9    Unprotected write to device memory     DRM_IOCTL_XOCL_PWRITE_UNMGD    drm_xocl_pwrite_unmgd
- * 10   Unprotected read from device memory    DRM_IOCTL_XOCL_PREAD_UNMGD     drm_xocl_pread_unmgd
- * 11   Send an execute job to a compute unit  DRM_IOCTL_XOCL_EXECBUF         drm_xocl_execbuf
- * 12   Register eventfd handle for MSIX       DRM_IOCTL_XOCL_USER_INTR       drm_xocl_user_intr
+ * 9    Create a hw context on a slot for a    DRM_IOCTL_XOCL_CREATE_HW_CTX   drm_xocl_create_hw_ctx
+ *      xclbin on the device
+ * 10   Destroy a hw context on a slot for a   DRM_IOCTL_XOCL_DESTROY_HW_CTX  drm_xocl_destroy_hw_ctx
+ *      xclbin on the device
+ * 11   Open a context on a compute unit       DRM_IOCTL_XOCL_OPEN_CU_CTX     drm_xocl_open_cu_ctx
+ *      on the device
+ * 12   Close a context on a compute unit      DRM_IOCTL_XOCL_CLOSE_CU_CTX    drm_xocl_close_cu_ctx
+ *      on the device
+ * 13   Unprotected write to device memory     DRM_IOCTL_XOCL_PWRITE_UNMGD    drm_xocl_pwrite_unmgd
+ * 14   Unprotected read from device memory    DRM_IOCTL_XOCL_PREAD_UNMGD     drm_xocl_pread_unmgd
+ * 15   Send an execute job to a compute unit  DRM_IOCTL_XOCL_EXECBUF         drm_xocl_execbuf
+ * 16   Send an execute job to a compute unit  DRM_IOCTL_XOCL_HW_CTX_EXECBUF  drm_xocl_hw_ctx_execbuf
+	under a specific hw context
+ * 17   Register eventfd handle for MSIX       DRM_IOCTL_XOCL_USER_INTR       drm_xocl_user_intr
  *      interrupt
- * 13   Update device view with a specific     DRM_IOCTL_XOCL_READ_AXLF       drm_xocl_axlf
+ * 18   Update device view with a specific     DRM_IOCTL_XOCL_READ_AXLF       drm_xocl_axlf
  *      xclbin image
- * 14   Obtain info of bo                      DRM_IOCTL_XOCL_INFO            drm_xocl_info_bo
- * 15   Obtain bo related statistics           DRM_IOCTL_XOCL_OCL_USAGE_STAT  drm_xocl_usage_stat
- * 16   Perform hot reset                      DRM_IOCTL_XOCL_HOT_RESET       N/A
- * 17   Perform clock scaling                  DRM_IOCTL_XOCL_RECLOCK         drm_xocl_reclock_info
- * 18   Allocate buffer on host memory         DRM_IOCTL_XOCL_ALLOC_CMA       drm_xocl_alloc_cma_info
- * 19   Free host memory buffer                DRM_IOCTL_XOCL_FREE_CMA        N/A
- * 20   Copy bo buffers                        DRM_IOCTL_XOCL_COPY_BO         drm_xocl_copy_bo
+ * 19   Obtain info of bo                      DRM_IOCTL_XOCL_INFO            drm_xocl_info_bo
+ * 20   Obtain bo related statistics           DRM_IOCTL_XOCL_OCL_USAGE_STAT  drm_xocl_usage_stat
+ * 21   Perform hot reset                      DRM_IOCTL_XOCL_HOT_RESET       N/A
+ * 22   Perform clock scaling                  DRM_IOCTL_XOCL_RECLOCK         drm_xocl_reclock_info
+ * 23   Allocate buffer on host memory         DRM_IOCTL_XOCL_ALLOC_CMA       drm_xocl_alloc_cma_info
+ * 24   Free host memory buffer                DRM_IOCTL_XOCL_FREE_CMA        N/A
+ * 25   Copy bo buffers                        DRM_IOCTL_XOCL_COPY_BO         drm_xocl_copy_bo
  * ==== ====================================== ============================== ==================================
  */
 
@@ -141,6 +152,14 @@ enum drm_xocl_ops {
 	DRM_XOCL_OCL_RESET,
 	/* Open/close a context */
 	DRM_XOCL_CTX,
+	/* Create a hw context */
+	DRM_XOCL_CREATE_HW_CTX,
+	/* Destroy a hw context */
+	DRM_XOCL_DESTROY_HW_CTX,
+	/* Open a cu context */
+	DRM_XOCL_OPEN_CU_CTX,
+	/* Close a cu context */
+	DRM_XOCL_CLOSE_CU_CTX,
 	/* Get information from device */
 	DRM_XOCL_INFO,
 	/* Unmanaged DMA from/to device */
@@ -150,6 +169,8 @@ enum drm_xocl_ops {
 	DRM_XOCL_USAGE_STAT,
 	/* Command to run on one or more CUs */
 	DRM_XOCL_EXECBUF,
+	/* Command to run on one or more CUs under hw context */
+	DRM_XOCL_HW_CTX_EXECBUF,
 	/* Register eventfd for user interrupts */
 	DRM_XOCL_USER_INTR,
 	/* Read xclbin/axlf */
@@ -164,6 +185,8 @@ enum drm_xocl_ops {
 	DRM_XOCL_FREE_CMA,
 	/* Memory to Memory BO copy */
 	DRM_XOCL_COPY_BO,
+	/* Set CU read-only range */
+	DRM_XOCL_SET_CU_READONLY_RANGE,
 
 	/* The following IOCTLs can only be called from linux kernel space
 	 * WARNING: INTERNAL USE ONLY. NOT FOR PUBLIC CONSUMPTION.
@@ -195,6 +218,10 @@ enum drm_xocl_sync_bo_dir {
 #define DRM_XOCL_NUM_SUPPORTED_CLOCKS	4
 
 #define DRM_XOCL_CMA_CHUNK_MAX		4
+
+#define CU_NAME_MAX_LEN 		64
+#define MAX_DEPENT_CMD_BO		8
+
 /**
  * struct drm_xocl_create_bo - Create buffer object
  * used with DRM_IOCTL_XOCL_CREATE_BO ioctl
@@ -311,7 +338,7 @@ struct drm_xocl_sync_bo_cb {
  * used with DRM_IOCTL_XOCL_INFO_BO IOCTL
  *
  * @handle:	bo handle
- * @flags:      Unused
+ * @flags:	Flags passed while creating BO (out)
  * @size:	Size of buffer object (out)
  * @paddr:	Physical address (out)
  */
@@ -340,6 +367,20 @@ struct drm_xocl_copy_bo {
 	uint64_t src_offset;
 };
 
+/**
+ * struct drm_xocl_set_cu_range - Set CU range
+ * used with DRM_IOCTL_XOCL_SET_CU_READONLY_RANGE
+ *
+ * @cu_index:	Index of the compute unit in the device inage for which
+ * @start:	Start offset of the range
+ * @size:	Size of the range
+ */
+struct drm_xocl_set_cu_range {
+	uint32_t cu_index;
+	uint32_t start;
+	uint32_t size;
+};
+
 /**
  * struct argument_info - Kernel argument information
  *
@@ -349,22 +390,28 @@ struct drm_xocl_copy_bo {
  * @dir:	input or output argument for a CU
  */
 struct argument_info {
-	char		name[32];
+	char		name[64];
 	uint32_t	offset;
 	uint32_t	size;
 	uint32_t	dir;
 };
 
+/* Kernel features macro */
+#define KRNL_SW_RESET	(1 << 0)
 /**
  * struct kernel_info - Kernel information
  *
  * @name:	kernel name
+ * @range:	kernel register range
  * @anums:	number of argument
+ * @features:	number of argument
  * @args:	argument array
  */
 struct kernel_info {
 	char			 name[64];
+	uint32_t		 range;
 	int			 anums;
+	int			 features;
 	struct argument_info	 args[];
 };
 
@@ -388,6 +435,38 @@ struct drm_xocl_kinfo_bo {
 	uint64_t vaddr;
 };
 
+/**
+ * struct drm_xocl_kds - KDS user configuration
+ *
+ * @slot_size:	CQ slot size
+ * @ert:	enable embedded HW scheduler
+ * @polling:	poll for command completion
+ * @cu_dma:	enable CUDMA custom module for HW scheduler
+ * @cu_isr:	enable CUISR custom module for HW scheduler
+ * @cq_int:	enable interrupt from host to HW scheduler
+ * @dataflow:	enable dataflow mode
+ * @rw_shared:	allow xclRegWrite/xclRegRead access shared CU
+ */
+struct drm_xocl_kds {
+	uint32_t slot_size;
+	uint32_t ert:1;
+	uint32_t polling:1;
+	uint32_t cu_dma:1;
+	uint32_t cu_isr:1;
+	uint32_t cq_int:1;
+	uint32_t dataflow:1;
+	uint32_t rw_shared:1;
+	uint32_t unused:25;
+};
+
+/*
+ * enum drm_xocl_axlf_flags - used for axlf programming
+ */
+enum drm_xocl_axlf_flags {
+	XOCL_AXLF_BASE		        = 0,
+	XOCL_AXLF_FORCE_PROGRAM		= (1 << 0)
+};
+
 /**
  * struct drm_xocl_axlf - load xclbin (AXLF) device image
  * used with DRM_IOCTL_XOCL_READ_AXLF ioctl
@@ -396,11 +475,15 @@ struct drm_xocl_kinfo_bo {
  * @xclbin:	Pointer to user's xclbin structure in memory
  * @ksize:	size of kernels in bytes
  * @kernels:	pointer of argument array
+ * @kds_cfg:	kds configuration
+ * @flags:	flags passed while programming xclbin
  */
 struct drm_xocl_axlf {
 	struct axlf		*xclbin;
 	int			 ksize;
 	char			*kernels;
+	struct drm_xocl_kds	 kds_cfg;
+	uint32_t		 flags;
 };
 
 /**
@@ -441,7 +524,8 @@ struct drm_xocl_pread_bo {
 
 enum drm_xocl_ctx_code {
 	XOCL_CTX_OP_ALLOC_CTX = 0,
-	XOCL_CTX_OP_FREE_CTX
+	XOCL_CTX_OP_FREE_CTX,
+	XOCL_CTX_OP_OPEN_UCU_FD,
 };
 
 #define	XOCL_CTX_SHARED		0x0
@@ -467,6 +551,66 @@ struct drm_xocl_ctx {
 	uint32_t handle;
 };
 
+/**
+ * struct drm_xocl_create_hw_ctx - Create a hw context on a slot on device
+ * used with DRM_XOCL_CREATE_HW_CTX ioctl
+ *
+ * @axlf_ptr:      axlf pointer which need to download
+ * @qos:           QOS information
+ * @hw_context:    Returns Context handle
+ */
+struct drm_xocl_create_hw_ctx {
+	struct drm_xocl_axlf	*axlf_ptr;
+	uint32_t		qos;
+
+	// return context id
+	uint32_t		hw_context;
+};
+
+/**
+ * struct drm_xocl_destroy_hw_ctx - Close/Destroy a hw context on a slot on device
+ * used with DRM_XOCL_DESTROY_HW_CTX ioctl
+ *
+ * @hw_context:    Context handle which need to close
+ */
+struct drm_xocl_destroy_hw_ctx {
+	uint32_t	hw_context;
+};
+
+/**
+ * struct drm_xocl_open_cu_ctx - Open a cu context under a hw context on device
+ * used with DRM_XOCL_OPEN_CU_CTX ioctl
+ *
+ * @hw_context:    Open CU under this hw Context handle
+ * @cu_name:       Name of the compute unit in the device image for which
+ * 		   the open request is being made
+ * @flags:         Shared or exclusive context (XOCL_CTX_SHARED/XOCL_CTX_EXCLUSIVE)
+ * @cu_index:      Return the acquired CU index. This will require for close cu context
+ */
+struct drm_xocl_open_cu_ctx {
+	// Under this hw context id
+	uint32_t	hw_context;
+	char		cu_name[CU_NAME_MAX_LEN];
+	uint32_t	flags;
+
+	// Return the acquired CU index.
+	uint32_t	cu_index;
+};
+
+/**
+ * struct drm_xocl_close_cu_ctx - Open a cu context under a hw context on device
+ * used with DRM_XOCL_CLOSE_CU_CTX ioctl
+ *
+ * @hw_context:    Open CU under this hw Context handle
+ * @cu_index:      Index of the compute unit in the device image for which
+ *                 the close request is being made
+ */
+struct drm_xocl_close_cu_ctx {
+	// Under this hw context id
+	uint32_t	hw_context;
+	uint32_t	cu_index;
+};
+
 struct drm_xocl_info {
 	unsigned short vendor;
 	unsigned short device;
@@ -517,6 +661,7 @@ struct drm_xocl_pread_unmgd {
 
 
 struct drm_xocl_mm_stat {
+	bool 	is_used;
 	size_t memory_usage;
 	unsigned int bo_count;
 };
@@ -554,13 +699,28 @@ enum drm_xocl_execbuf_state {
  *
  * @ctx_id:         Pass 0
  * @exec_bo_handle: BO handle of command buffer formatted as ERT command
- * @deps:	    Upto 8 dependency command BO handles this command is dependent on
+ * @deps:	    Upto MAX_DEPENT_CMD_BO dependency command BO handles this command is dependent on
  *                  for automatic event dependency handling by ERT
  */
 struct drm_xocl_execbuf {
 	uint32_t ctx_id;
 	uint32_t exec_bo_handle;
-	uint32_t deps[8];
+	uint32_t deps[MAX_DEPENT_CMD_BO];
+};
+
+/**
+ * struct drm_xocl_hw_ctx_execbuf - Submit a command buffer for execution on a compute unit
+ * used with DRM_IOCTL_XOCL_HW_CTX_EXECBUF ioctl with new hw context parameters
+ *
+ * @hw_ctx_id:	    Pass the HW Context id
+ * @exec_bo_handle: BO handle of command buffer formatted as ERT command
+ * @deps:	    Upto MAX_DEPENT_CMD_BO dependency command BO handles this command is dependent on
+ *                  for automatic event dependency handling by ERT
+ */
+struct drm_xocl_hw_ctx_execbuf {
+	uint32_t hw_ctx_id;
+	uint32_t exec_bo_handle;
+	uint32_t deps[MAX_DEPENT_CMD_BO];
 };
 
 /**
@@ -638,18 +798,24 @@ struct drm_xocl_alloc_cma_info {
 #define	DRM_IOCTL_XOCL_PWRITE_BO	XOCL_IOC_ARG(PWRITE_BO, pwrite_bo)
 #define	DRM_IOCTL_XOCL_PREAD_BO		XOCL_IOC_ARG(PREAD_BO, pread_bo)
 #define	DRM_IOCTL_XOCL_CTX		XOCL_IOC_ARG(CTX, ctx)
+#define	DRM_IOCTL_XOCL_CREATE_HW_CTX	XOCL_IOC_ARG(CREATE_HW_CTX, create_hw_ctx)
+#define	DRM_IOCTL_XOCL_DESTROY_HW_CTX	XOCL_IOC_ARG(DESTROY_HW_CTX, destroy_hw_ctx)
+#define	DRM_IOCTL_XOCL_OPEN_CU_CTX	XOCL_IOC_ARG(OPEN_CU_CTX, open_cu_ctx)
+#define	DRM_IOCTL_XOCL_CLOSE_CU_CTX	XOCL_IOC_ARG(CLOSE_CU_CTX, close_cu_ctx)
 #define	DRM_IOCTL_XOCL_INFO		XOCL_IOC_ARG(INFO, info)
 #define	DRM_IOCTL_XOCL_READ_AXLF	XOCL_IOC_ARG(READ_AXLF, axlf)
 #define	DRM_IOCTL_XOCL_PWRITE_UNMGD	XOCL_IOC_ARG(PWRITE_UNMGD, pwrite_unmgd)
 #define	DRM_IOCTL_XOCL_PREAD_UNMGD	XOCL_IOC_ARG(PREAD_UNMGD, pread_unmgd)
 #define	DRM_IOCTL_XOCL_USAGE_STAT	XOCL_IOC_ARG(USAGE_STAT, usage_stat)
 #define	DRM_IOCTL_XOCL_EXECBUF		XOCL_IOC_ARG(EXECBUF, execbuf)
+#define	DRM_IOCTL_XOCL_HW_CTX_EXECBUF	XOCL_IOC_ARG(HW_CTX_EXECBUF, hw_ctx_execbuf)
 #define	DRM_IOCTL_XOCL_USER_INTR	XOCL_IOC_ARG(USER_INTR, user_intr)
 #define	DRM_IOCTL_XOCL_HOT_RESET	XOCL_IOC(HOT_RESET)
 #define	DRM_IOCTL_XOCL_RECLOCK		XOCL_IOC_ARG(RECLOCK, reclock_info)
 #define	DRM_IOCTL_XOCL_ALLOC_CMA	XOCL_IOC_ARG(ALLOC_CMA, alloc_cma_info)
 #define	DRM_IOCTL_XOCL_FREE_CMA		XOCL_IOC(FREE_CMA)
 #define	DRM_IOCTL_XOCL_COPY_BO		XOCL_IOC_ARG(COPY_BO, copy_bo)
+#define	DRM_IOCTL_XOCL_SET_CU_READONLY_RANGE	XOCL_IOC_ARG(SET_CU_READONLY_RANGE, set_cu_range)
 
 #define	DRM_IOCTL_XOCL_KINFO_BO		XOCL_IOC_ARG(KINFO_BO, kinfo_bo)
 #define	DRM_IOCTL_XOCL_MAP_KERN_MEM	XOCL_IOC_ARG(MAP_KERN_MEM, map_kern_mem)
