@@ -1,5 +1,6 @@
 /**
- * Copyright (C) 2020 Xilinx, Inc
+ *  Copyright (C) 2020 Xilinx, Inc
+ *  Copyright (C) 2022-2025 Advanced Micro Devices, Inc. - All rights reserved
  *
  * Licensed under the Apache License, Version 2.0 (the "License"). You may
  * not use this file except in compliance with the License. A copy of the
@@ -17,12 +18,20 @@
 #ifndef AIE_PROFILE_DOT_H
 #define AIE_PROFILE_DOT_H
 
-namespace xdpaieprofile {
+namespace xdp::aie {
+  namespace profile {
 
-  void load_xdp_aie_plugin();
-  void register_aie_callbacks(void* handle);
-  void warning_aie_callbacks();
+    void load();
+    void register_callbacks(void* handle);
+    void warning_callbacks();
 
-} // end namespace xdpaieprofile
+  } // end namespace profile
+
+  namespace ctr {
+    void update_device(void* handle, bool hw_context_flow);
+    void end_poll(void* handle);
+  } // end namespace ctr
+
+} // end namespace xdp::aie
 
 #endif
